from pathlib import Path

from . import names as const_names

"""
DEFAULT PATHS
"""

# path list under root directory
BASE_DIR_PATH = Path(Path.cwd().__str__().split(const_names.RESOURCES_DIR_NAME)[0])
APP_DIR_PATH = BASE_DIR_PATH.joinpath(const_names.APP_DIR_NAME)
WRITABLE_DIR_PATH = BASE_DIR_PATH.joinpath(const_names.WRITABLE_DIR_NAME)
RESOURCE_DIR_PATH = BASE_DIR_PATH.joinpath(const_names.RESOURCES_DIR_NAME)

PROD_ENV_FILE_PATH = BASE_DIR_PATH.joinpath(const_names.PROD_ENV_FILE_NAME)
DEV_ENV_FILE_PATH = BASE_DIR_PATH.joinpath(const_names.DEV_ENV_FILE_NAME)

# path list under writable directory
LOGS_DIR_PATH = WRITABLE_DIR_PATH.joinpath(const_names.LOGS_DIR_NAME)

# path list under resources directory
FIXTURES_DIR_PATH = RESOURCE_DIR_PATH.joinpath(const_names.FIXTURES_DIR_NAME)
RESOURCE_CONFIG_DIR_PATH = RESOURCE_DIR_PATH.joinpath(const_names.RESOURCE_CONFIG_DIR_NAME)
SETUP_SCRIPTS_DIR_PATH = RESOURCE_DIR_PATH.joinpath(const_names.SETUP_SCRIPTS_DIR_NAME)
TEST_DATA_DIR_PATH = RESOURCE_DIR_PATH.joinpath(const_names.TEST_DATA_DIR_NAME)

# path list under fixtures directory
BASE_FIXTURES_DIR_PATH = FIXTURES_DIR_PATH.joinpath(const_names.DEFAULT_FIXTURE_DIR_NAME),
PROD_FIXTURES_DIR_PATH = FIXTURES_DIR_PATH.joinpath(const_names.PROD_FIXTURE_DIR_NAME),
TEST_FIXTURES_DIR_PATH = FIXTURES_DIR_PATH.joinpath(const_names.TEST_FIXTURE_DIR_NAME),

# compiled constants
SYS_PATH_LIST = [
	BASE_DIR_PATH.__str__(), APP_DIR_PATH.__str__(),
	RESOURCE_DIR_PATH.__str__(), WRITABLE_DIR_PATH.__str__()
]

FIXTURE_DIRS_LIST = [BASE_FIXTURES_DIR_PATH, PROD_FIXTURES_DIR_PATH, TEST_FIXTURES_DIR_PATH]



from ._client import (ApiRequestClient)
from ._wrapper import (ApiRouter, MsbIntraServiceRequest)
from ._request import (RequestHeaders, RequestInfo)
from ._response import (ApiResponse, RestResponse)
from ._dataclasses import (ApiResponseWrapper, RequestWrapper, HostUrlsConfig, ApiRequestData, MsbApiResponse)
from ._utils import make_api_request
__all__ = [
	"ApiResponse", "ApiRouter", "ApiRequestData", "ApiResponseWrapper", "MsbApiResponse",
	"RequestInfo", "RequestHeaders", "RequestWrapper", "MsbIntraServiceRequest", "RestResponse",
	"HostUrlsConfig","make_api_request",
]



from ._constants import *
from ._defaults import (jwt_user_auth_rule, DefaultJwtAuthSettings)
from .permissions import (LoginRequiredPermission, AdminUserPermission)
from .results import AuthResult
from .users import (TokenUser)



"""
CONSTANT DIRECTORY NAMES
"""

APP_DIR_NAME = "app"
RESOURCES_DIR_NAME = "resources"
WRITABLE_DIR_NAME = "writable"
FIXTURES_DIR_NAME = "fixtures"

DEFAULT_FIXTURE_DIR_NAME = "base"
PROD_FIXTURE_DIR_NAME = "prod"
TEST_FIXTURE_DIR_NAME = "test"

RESOURCE_CONFIG_DIR_NAME = "config"
SETUP_SCRIPTS_DIR_NAME = "setup"
TEST_DATA_DIR_NAME = "test_data"
LOGS_DIR_NAME = "fixtures"
MIGRATIONS_DIR_NAME = "migrations"
"""
CONSTANT ENVIRONMENT NAMES
"""
PROD_ENV_NAME = "prod"
STAGE_ENV_NAME = "stage"
TEST_ENV_NAME = "test"
DEV_ENV_NAME = "dev"
LOCAL_ENV_NAME = "local"

ENV_NAME_LIST = [LOCAL_ENV_NAME, DEV_ENV_NAME, TEST_ENV_NAME, STAGE_ENV_NAME, PROD_ENV_NAME]
DEV_OR_TEST_ENV_NAMES_LIST = [LOCAL_ENV_NAME, DEV_ENV_NAME, TEST_ENV_NAME]
"""
CONSTANT FILE NAMES
"""
PROD_ENV_FILE_NAME = ".env"
DEV_ENV_FILE_NAME = ".env.local"

"""
CONSTANT DATABASE NAMES
"""
DEFAULT_DATABASE_NAME = "default"
LOGS_DATABASE_NAME = "logs"

YAML_FILE_EXTENTION_NAME = "yaml"


from typing import (List, Dict, Self)

from requests.models import Response

from ._dataclasses import (ApiRequestData, ApiResponseWrapper)
from ._utils import make_api_request


class ApiRequestClient(ApiRequestData):

	@property
	def request_is_json(self) -> bool:
		return True

	def GET(self, query_params: [List | Dict] = None, ) -> Self:
		return self.set_query_params(params=query_params).set_request_method("GET")

	def POST(self, data: [List | Dict] = None, query_params: [List | Dict] = None) -> Self:
		return self.set_data(data=data).set_query_params(params=query_params).set_request_method("POST")

	def PUT(self, data: [List | Dict] = None, query_params: [List | Dict] = None) -> Self:
		return self.set_data(data=data).set_query_params(params=query_params).set_request_method("PUT")

	def DELETE(self, data: [List | Dict] = None, query_params: [List | Dict] = None) -> Self:
		return self.set_data(data=data).set_query_params(params=query_params).set_request_method("DELETE")

	def get_result(self) -> ApiResponseWrapper:
		try:
			_response = make_api_request(self)
		except Exception as e:
			_response = ApiResponseWrapper(Response())
		return _response



from . import names as const_names
from . import paths as const_paths
from . import regex as const_regex
from . import http as const_http

__all__ = ["const_names", "const_paths", "const_regex","const_http"]



from json import dump as json_dump
from typing import (List, Dict)

from ._client import ApiRequestClient
from ._dataclasses import (ApiRequestData, MsbApiResponse, RestRequest)


class ApiRouter(ApiRequestData):

	def _get_payload(self, request: RestRequest):
		if self.method.lower() in ['get']:
			return None

		return json_dump((request.data if type(request.data) in [list, dict] else {}))

	def __init__(self, request: RestRequest):
		super(ApiRouter, self).__init__(request=request)
		self.set_request_method(request.method)
		self.set_data(self._get_payload(request))


class MsbIntraServiceRequest:
	service_host: str = ''
	__api_request: ApiRequestClient

	def __init__(self, request: RestRequest = None, version: int = 1):
		self.__api_request = ApiRequestClient(api_host=self.service_host, request=request)

	def __execute(self):
		api_response = self.__api_request.get_result()
		return api_response.as_msb_api_response()

	@property
	def request(self) -> ApiRequestClient:
		return self.__api_request

	def retrieve(self, endpoint: str, pk: str) -> MsbApiResponse:
		self.__api_request.set_endpoint(endpoint=endpoint).GET(query_params=[pk])
		return self.__execute()

	def list(self, endpoint: str, limit: int = -1, offset: int = 0) -> MsbApiResponse:
		self.__api_request.set_endpoint(endpoint=endpoint).GET(query_params=dict(limit=limit, offset=offset))
		return self.__execute()

	def create(self, endpoint: str, data: [List | Dict] = None) -> MsbApiResponse:
		self.__api_request.set_endpoint(endpoint=endpoint).POST(data=data)
		return self.__execute()

	def search(self, endpoint: str, params: dict) -> MsbApiResponse:
		endpoint = f"{endpoint.strip('/')}/search"
		data = params.__dict__
		self.__api_request.set_endpoint(endpoint=endpoint).POST(data=data)
		return self.__execute()

	def update(self, endpoint: str, pk: str, data: Dict = None) -> MsbApiResponse:
		self.__api_request.set_endpoint(endpoint=endpoint).PUT(data=data, query_params=[pk])
		return self.__execute()

	def bulk_update(self, endpoint: str, data: List[Dict] = None) -> MsbApiResponse:
		self.__api_request.set_endpoint(endpoint=endpoint).PUT(data=data)
		return self.__execute()

	def delete(self, endpoint: str, pk: str) -> MsbApiResponse:
		self.__api_request.set_endpoint(endpoint=endpoint).POST(query_params=[pk])
		return self.__execute()

	def bulk_delete(self, endpoint: str, data: List[Dict]) -> MsbApiResponse:
		self.__api_request.set_endpoint(endpoint=endpoint).POST(data=data)
		return self.__execute()
